******************************************************************************
*
*       Function:   PROC_READY
*       Usage:      Called as an AST service routine
*       Purpose:    This routine is used to determine when a subprocess is
*                   ready for work.  It is used as an AST routine to a
*                   READATTN request on the mailbox associated with a
*                   subprocesses SYS$INPUT.  After a START_TASK request,
*                   this routine should be called after a newly created
*                   subprocess has started, or immediately after the
*                   READATTN request for a preexisting subprocess.  This 
*                   routine tells the JBC that the task has started, and
*                   initiates the processing of the job by writing the
*                   appropriate information down the mailbox to the process.
*                   It then ask for another READATTN on the mailbox so that
*                   we will know when the process finishes, and asks for more
*                   INPUT (No, this is not Number 5).
*       Output:     A running subprocess that is performing the job requested,
*                   a pending AST request for notification that the process
*                   has completed its task, and notification to the JBC that
*                   the task has started.
*
******************************************************************************

        subroutine proc_ready (proc)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            record /sub_proc/ proc
            integer clen, len
            integer*2 wlen
            character*(BUFSIZ) cmd, proc_file
            external reply, signal, proc_done, blength, prcnam, log_sys
            integer blength, stream, status
            character*2 prcnam

*           Check to see that the I/O completed OK.

            if ((proc.iosb.cond .and. 1).ne.1) then
                call log_sys (%val (proc.iosb.cond), 
     .                              'Error in $QIO waiting for task to start')
                status = proc.iosb.cond
                call reply (status, SMBMSG$K_TASK_COMPLETE, proc.stream)
                proc.avail = .true.
                proc.started = .false.
                proc.starting = .false.
                proc.ready = .false.
                proc.dying = .false.
                proc.dedicated = .false.
                return
            endif

            stream = proc.stream

*           If this process is not special, then assure the requested priority

            if (.not. streams(stream).special) then

*               Set the priority to the proper value.

                cmd = 'SET PROC/PRIO='//streams(stream).proc_pri
                clen = 17
                call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(clen), , , , ))
            endif

*           Change the process name.

            clen = blength (streams(stream).exec_queue)
            if (clen .gt. 11) clen = 11
            cmd = 'SET PROC/NAME="'//
     .                      streams(stream).exec_queue(1:clen)
     .                      // '_' //prcnam(proc.idx)//'"'
            clen = clen + 20

            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(clen), , , , ))

*           Assign the symbols for the queue names

            len = blength (streams(stream).queue)
            cmd = 'SMB$QUEUE=="'//streams(stream).queue(1:len)//'"'
            clen = len + 13

            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(clen), , , , ))

            len = blength (streams(stream).exec_queue)
            cmd = 'SMB$EQUEUE=="'//streams(stream).exec_queue(1:len)//'"'
            clen = len + 14

            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(clen), , , , ))

*           Pass the entry number.

            call signal (sys$fao (
     .              'SMB$ENTRY==!UL',
     .              wlen, %descr(cmd),
     .              %val (proc.entry)))

            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(wlen), , , , ))

*           Pass the UIC of the submitting process (may not be same as username)

            call signal (sys$fao (
     .              'SMB$UIC=="!%I"',
     .              wlen, %descr(cmd),
     .              %val (streams(stream).sub_uic)))

            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(wlen), , , , ))

*           Pass the submitting username.

            wlen = blength(streams(stream).sub_user)
            call signal (sys$fao (
     .              'SMB$USERNAME=="!AS"',
     .              wlen, %descr(cmd),
     .              %descr (streams(stream).sub_user(1:wlen))))

            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(wlen), , , , ))

*           If the file= qualifier is used, then always submit that file,
*           and pass the file specified as an argument.

            if (streams (stream).proc_spec(1:1) .ne. ' ') then
                len = blength(streams (stream).proc_spec)
                proc_file = streams (stream).proc_spec(1:len)

*               Pass the file spec we are supposed to envoke.

                cmd = 'SMB$FILESPEC=="'// streams (stream).fspec(1:
     .              blength(streams (stream).fspec)) // '"'
                clen = blength(cmd)
            else
                len = blength(streams (stream).fspec)
                proc_file = streams (stream).fspec(1:len)

*               Nullify any previous version of this symbol

                cmd = 'SMB$FILESPEC==""'
                clen = blength(cmd)
            endif

*           Send out the assignment command

            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(clen), , , , ))

*           Build the command file invocation command line.

            call signal (sys$fao (
     .              '@!AS "!AS" "!AS" "!AS" "!AS" "!AS" "!AS" "!AS" "!AS"',
     .              wlen, %descr(cmd),
     .              proc_file(1:len),
     .              proc.param(1)(1:proc.paramlen(1)),
     .              proc.param(2)(1:proc.paramlen(2)),
     .              proc.param(3)(1:proc.paramlen(3)),
     .              proc.param(4)(1:proc.paramlen(4)),
     .              proc.param(5)(1:proc.paramlen(5)),
     .              proc.param(6)(1:proc.paramlen(6)),
     .              proc.param(7)(1:proc.paramlen(7)),
     .              proc.param(8)(1:proc.paramlen(8))
     .              ))

*           Send the DCL command to start the command file

            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb, , ,
     .                      %ref(cmd),
     .                      %val(wlen), , , , ))
                
*           Set the process ready, and tell the JBC it is starting

            proc.ready = .true.
            call reply (SS$_NORMAL, SMBMSG$K_START_TASK, stream)

*           Reset the READATTN to go to a function to signal completion.

            call get_read_attn (proc, proc_done)
        end
